/*
  Jind - an interpreter for a toy programming language
  Copyright (C) 2000, 2001 Markus Triska

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA

  Contact me at triska@gmx.at
*/

import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;


public class Jind extends Applet implements ActionListener
{

    final int WIDTH =  600;
    final int HEIGHT = 450;

    TextArea input = new TextArea("# Enter instructions here", 15, 40);
    Button BT_start = new Button("Start");
    Game game = null;
    Position oldplayerpos = new Position(1, 1);
    boolean interpreting = false;
    boolean boarddrawn = false;

    Button BT_step = new Button("Step");
    Button BT_run = new Button("Run");
    Button BT_close = new Button("Close");

    Graphics temp_screen;
    Image temp_image;

    public Jind()
    {

    }

    public String getAppletInfo()
    {
	return "Name: Jind\r\n" +
	    "Author: Markus Triska triska@gmx.at\r\n" +
	    "Date released: Nov. 3rd 2001";
    }

    public void init()
    {
	add(input);
	add(BT_start);
	BT_start.addActionListener(this);
	BT_close.addActionListener(this);
	BT_step.addActionListener(this);
	BT_run.addActionListener(this);

	BT_close.setVisible(false);
	BT_step.setVisible(false);
	BT_run.setVisible(false);

	add(BT_step);
	// add(BT_run);
	add(BT_close);

	resize(WIDTH, HEIGHT);
    }

    public void destroy()
    {

    }

    public void draw_board()
    {
	temp_image = createImage(WIDTH, HEIGHT);
	temp_screen = temp_image.getGraphics();
	temp_screen.setColor(Color.gray);
	temp_screen.fillRect(0, 0, WIDTH, HEIGHT);
	int curposx = 0;
	int curposy = 0;
	game.field.startIteration();
	while (game.field.nextRow()) {
	    while (game.field.nextCol()) {
		if (game.field.getPiece() == Globals.BORDER) {
		    temp_screen.setColor(Color.black);
		} else if (game.field.getPiece() == Globals.BLOCK) {
		    temp_screen.setColor(Color.blue);
		} else if (game.field.getPiece() == Globals.FREE) {
		    temp_screen.setColor(Color.white);
		}
		temp_screen.fill3DRect(curposx*10 + 50, curposy*10 + 50, 10, 10, true);
		curposx += 1;
	    }

	    curposy += 1;
	    curposx = 0;
	}

	temp_screen.setColor(Color.red);
	temp_screen.fill3DRect(490 + 50, 190 + 50, 10, 10, true);

	boarddrawn = true;
    }

    public void paint(Graphics g)
    {

	if (interpreting) {
	    if (!boarddrawn) { draw_board(); }

	    temp_screen.setColor(Color.white);
	    temp_screen.fill3DRect(oldplayerpos.x * 10 + 50, oldplayerpos.y * 10 + 50, 10, 10, true);
	    temp_screen.setColor(Color.black);
	    int x = game.player.position().x * 10 + 50;
	    int y = game.player.position().y * 10 + 50;
	    switch (game.player.direction()) {
	    case 1:
		temp_screen.drawLine(x, y + 9 , x + 5, y);
		temp_screen.drawLine(x + 9, y + 9, x + 5, y);
		break;
	    case 2:
		temp_screen.drawLine(x, y, x + 9, y + 5);
		temp_screen.drawLine(x, y + 9, x + 9, y + 5);
		break;
	    case 3:
		temp_screen.drawLine(x, y, x + 5, y + 9);
		temp_screen.drawLine(x + 9, y, x + 5, y + 9);
		break;
	    case 4:
		temp_screen.drawLine(x + 9, y, x, y + 5);
		temp_screen.drawLine(x + 9, y + 9, x , y + 5);
		break;
	    }

	    if (temp_image != null) g.drawImage(temp_image, 0, 0, null);
	} else {
	    g.setColor(Color.gray);
	    g.fillRect(0, 0, WIDTH, HEIGHT);
	}
	g.setColor(Color.black);

	if (game != null && game.isError()) {
	    g.drawString(game.getErrorMessage(), 50, 400);
	}

	g.drawString("Copyright (C) 2000 Markus Triska (triska@gmx.at)", 50, 350);
	g.drawString("This program is dedicated to the most beautiful sea-horse on the whole world.", 50, 370);


    }

    public void start()
    {
    }

    public void stop()
    {
    }

    private void stop_interpretation()
    {

	BT_close.setVisible(false);
	BT_step.setVisible(false);
	BT_run.setVisible(false);
	interpreting = false;
	boarddrawn = false;
	BT_start.setVisible(true);
	input.setVisible(true);
	validate();
	repaint();
    }


    public void actionPerformed(ActionEvent event) {

	if (event.getSource() == BT_start) {
	    game = new Game("map.txt", input.getText());

	    BT_start.setVisible(false);
	    input.setVisible(false);

	    BT_step.setVisible(true);
	    BT_close.setVisible(true);
	    BT_run.setVisible(true);

	    interpreting = true;
	    validate();
	    boarddrawn = false;
	    repaint();
	} else if (event.getSource() == BT_close) {
	    stop_interpretation();
	} else if (event.getSource() == BT_step) {
	    oldplayerpos = game.player.position();
	    if (!game.next_statement()) stop_interpretation();

	    // Remove and redraw player
	    int x1 = oldplayerpos.x * 10 + 50;
	    int y1 = oldplayerpos.y * 10 + 50;
	    repaint(x1, y1, x1 + 10, y1 + 10);
	    int x2 = game.player.position().x * 10 + 50;
	    int y2 = game.player.position().y * 10 + 50;
	    repaint(x2, y2, x2 + 10, y2 + 10);

	} else if (event.getSource() == BT_run) {
	    oldplayerpos = game.player.position();
	    while (game.next_statement() && !game.won) {
		repaint();
		oldplayerpos = game.player.position();
		// sleep(200);
	    }
	    // stop_interpretation();
	}
    }
}
